<?php

namespace SM\Factory;

use Doctrine\ORM\EntityManager;
use SM\Traits\TQuery;

class Sm00006000ServicosFactory
{

    use TQuery;

    protected $connection;

    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    public function getInformacoesGridPrincipal()
    {

        $sql = "SELECT SERVICO.CDSERVORCA,
                        SERVICO.CDFILIAL,
                        SERVICO.NMSERVORCA,
                        SERVICO.DSSERVORCA,
                        SERVICO.DSSERVORCANF,
                        SERVICO.DSSERVORCABOL,
                        SERVICO.CDPRODFATSERV,
                        PRODUTO.NMPRODUTO,
                        SERVICO.IDTIPO,
                        SERVICO.NRORG,
                        SERVICO.DTINCLUSAO,
                        SERVICO.DTULTATU,
                        SERVICO.NRORGINCLUSAO,
                        SERVICO.CDOPERINCLUSAO,
                        OPINC.NMOPERADOR AS NMOPERINCLUSAO,
                        SERVICO.NRORGULTATU,
                        SERVICO.CDOPERULTATU,
                        OPATU.NMOPERADOR AS NMOPERULTATU,
                        SERVICO.IDATIVO
                    FROM FCT_SERVICOORCA SERVICO
                    LEFT JOIN PRODUTO
                    ON SERVICO.CDPRODFATSERV = PRODUTO.CDPRODUTO
                    AND SERVICO.NRORG         = PRODUTO.NRORG
                    LEFT JOIN OPERADOR OPINC
                    ON SERVICO.CDOPERINCLUSAO = OPINC.CDOPERADOR
                    AND SERVICO.NRORG          = OPINC.NRORG
                    LEFT JOIN OPERADOR OPATU
                    ON SERVICO.CDOPERULTATU = OPATU.CDOPERADOR
                    AND SERVICO.NRORG        = OPATU.NRORG
                WHERE SERVICO.IDTIPO = 'S'
                ORDER BY NMSERVORCA ASC";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getProdutoFaturamento()
    {

        $sql = "SELECT CDPRODUTO, CDARVPROD, NMPRODUTO,  NRORG 
                        FROM PRODUTO 
                        WHERE CDNVPRODUTO = (SELECT MAX(CDNVPRODUTO) FROM PRODUTO)";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function validacao($servico, $habilidade, $nrorg)
    {
        $sql = "SELECT COUNT(*) AS EXISTE
                    FROM FCT_RELACSERVHABILI
                WHERE CDSERVORCA  = :CDSERVORCA
                AND CDHABILITSERV  IN (" . $habilidade . ")
                AND NRORG          = :NRORG";

        $params = array(
            "CDSERVORCA" => $servico,
            "NRORG" => $nrorg
        );
        return $this->connection->executeQuery($sql, $params)->fetchAllAssociative();
    }

    public function findHabilidades($filtro)
    {

        $sql = "SELECT  REL.NRRELACSERVHABILI,
                        REL.CDSERVORCA,
                        REL.CDHABILITSERV,
                        HAB.NMHABILITSERV
                FROM FCT_RELACSERVHABILI REL
                INNER JOIN FCT_HABILITSERV HAB ON REL.CDHABILITSERV = HAB.CDHABILITSERV
                WHERE CDSERVORCA = :CDSERVORCA";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $filtro);
    }

    public function deletarServicos($codigos)
    {
        $sql = "DELETE FROM FCT_SERVICOORCA
                 WHERE CDSERVORCA IN ($codigos)";

        $this->connection->executeStatement($sql);
    }

    public function removeRelacaoHabilidade($params)
    {
        $sql = "DELETE 
                FROM FCT_RELACSERVHABILI 
                WHERE NRRELACSERVHABILI = :NRRELACSERVHABILI";

        $this->connection->executeStatement($sql, $params);
    }

    public function adicionarServico($params)
    {
        $sql = "INSERT INTO FCT_SERVICOORCA(
                    CDSERVORCA, 
                    NRORG, 
                    IDATIVO, 
                    DTINCLUSAO, 
                    CDOPERINCLUSAO, 
                    NMSERVORCA, 
                    CDPRODFATSERV,
                    DSSERVORCA,
                    DSSERVORCANF,
                    DSSERVORCABOL,
                    DTULTATU,
                    CDOPERULTATU
                )
                VALUES(:CDSERVORCA, 
                    :NRORG, 
                    :IDATIVO, 
                    SYSDATE, 
                    :CDOPERINCLUSAO, 
                    :NMSERVORCA, 
                    :CDPRODFATSERV,
                    :DSSERVORCA,
                    :DSSERVORCANF,
                    :DSSERVORCABOL,
                    SYSDATE,
                    :CDOPERULTATU
                )";
        $this->connection->executeStatement($sql, $params);
    }

    public function editarServico($params)
    {

        $sql = "UPDATE FCT_SERVICOORCA
                    SET IDATIVO = :IDATIVO,
                        NMSERVORCA = :NMSERVORCA,
                        CDPRODFATSERV = :CDPRODFATSERV,
                        DSSERVORCA = :DSSERVORCA,
                        DSSERVORCANF = :DSSERVORCANF,
                        DSSERVORCABOL = :DSSERVORCABOL,
                        DTULTATU = SYSDATE,
                        NRORGULTATU = :NRORGULTATU,
                        CDOPERULTATU = :CDOPERULTATU
                    WHERE CDSERVORCA = :CDSERVORCA";

        $this->connection->executeStatement($sql, $params);
    }

    public function insertRelacaoHabilidade($params)
    {
        $sql = "INSERT INTO FCT_RELACSERVHABILI(
                    NRRELACSERVHABILI,
                    CDSERVORCA,
                    CDHABILITSERV,
                    NRORG,
                    IDATIVO,
                    DTINCLUSAO,
                    DTULTATU,
                    NRORGINCLUSAO,
                    NRORGULTATU,
                    CDOPERULTATU
                )
                VALUES(
                    :NRRELACSERVHABILI,
                    :CDSERVORCA ,
                    :CDHABILITSERV,
                    :NRORG,
                    :IDATIVO,
                    SYSDATE,
                    SYSDATE,
                    :NRORGINCLUSAO,
                    :NRORGULTATU,
                    :CDOPERULTATU
                )";
        $this->connection->executeStatement($sql, $params);
    }

    public function insertRelacServico($params)
    {
        $sql = "INSERT INTO FCT_RELACSERVICO(
                            NRRELACSERVICO, 
                            CDSERVORCA, 
                            CDSERVORCARELAC, 
                            CDTPMEDIDASERV, 
                            QTMEDIDASERV, 
                            NRORG, 
                            IDATIVO,
                            DTINCLUSAO,
                            DTULTATU,
                            NRORGINCLUSAO,
                            CDOPERINCLUSAO,
                            NRORGULTATU,
                            CDOPERULTATU
                        )
                    VALUES(:NRRELACSERVICO, 
                            :CDSERVORCA, 
                            :CDSERVORCARELAC, 
                            :CDTPMEDIDASERV, 
                            :QTMEDIDASERV,
                            :NRORG,
                            :IDATIVO,
                            SYSDATE,
                            SYSDATE,
                            :NRORGINCLUSAO,
                            :CDOPERINCLUSAO,
                            :NRORGULTATU,
                            :CDOPERULTATU
                        )";
        $this->connection->executeStatement($sql, $params);
    }

    public function validaServicoRelacionado($params)
    {

        $sql = "SELECT COUNT(*) AS VALIDASERV
                FROM  FCT_RELACSERVICO
                WHERE CDSERVORCARELAC IN (" . $params['CDSERVORCARELAC'] . ")
                AND CDTPMEDIDASERV = :CDTPMEDIDASERV
                AND CDSERVORCA = :CDSERVORCA
                AND NRORG = :NRORG";

        unset($params['CDSERVORCARELAC']);
        return $this->connection->executeQuery($sql, $params)->fetchAllAssociative();
    }

    public function getRelacServico($filtro)
    {

        $sql = "SELECT * FROM 
                (SELECT RELAC.* ,
                        TIPO.DSTPMEDIDASERV,
                        SERV.NMSERVORCA
                FROM FCT_RELACSERVICO RELAC
                INNER JOIN FCT_TIPOMEDIDASERV TIPO ON TIPO.CDTPMEDIDASERV = RELAC.CDTPMEDIDASERV
                INNER JOIN FCT_SERVICOORCA SERV ON SERV.CDSERVORCA = RELAC.CDSERVORCARELAC
                )WHERE CDSERVORCA = :CDSERVORCA";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $filtro);
    }

    public function deleteRelacServico($params)
    {
        $sql = "DELETE FROM FCT_RELACSERVICO WHERE NRRELACSERVICO = :NRRELACSERVICO";
        $this->connection->executeStatement($sql, $params);
    }
}
