<?php

namespace SM\Service;

use SM\Factory\CdaFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use SM\Service\FindService;
use SM\Service\ProcedureService;
use SM\Service\SmNextDocumentoCdaService;
use SM\Service\AssinaturaDigitalCDAService;
use Zeedhi\Framework\Log\Console;

class CdaService
{
    protected $CdaFactory;
    protected $Environment;
    protected $newCode;
    protected $connection;
    protected $ProcedureService;
    protected $DocumentoCdaService;
    protected $tempDirFS;

    public function __construct(EntityManager $entityManager, CdaFactory $CdaFactory, Environment $Environment, NewCode $newCode, ProcedureService $ProcedureService, SmNextDocumentoCdaService $DocumentoCdaService, AssinaturaDigitalCDAService $assinaturaCDAService)
    {
        $this->CdaFactory          = $CdaFactory;
        $this->connection          = $entityManager->getConnection();
        $this->Environment         = $Environment;
        $this->newCode             = $newCode;
        $this->ProcedureService    = $ProcedureService;
        $this->DocumentoCdaService = $DocumentoCdaService;
        $this->assinaturaCDAService = $assinaturaCDAService;

        $this->tempDirFS = sys_get_temp_dir();
        // $this->tempDirFS = '/home/developer/workfolder/anexos';
    }

    public function getOperadorLogado()
    {
        return array(
            "CDOPERADOR" => $this->Environment->getCdOperador(),
        );
    }

    public function getCda($params)
    {
        $data = $this->CdaFactory->getCda(array(
            "CDFUNC"            => $params["CDFUNC"],
            "CDFILIAL"          => $params["CDFILIAL"],
            "CDEMPRESA"         => $params["CDEMPRESA"],
            "NRORG"             => $params["NRORG"],
            "CDOPERADOR"        => $params["CDOPERADOR"],
            "DTINICIALPERIODO"  => $params["DTINICIALPERIODO"],
            "DTFINALPERIODO"    => $params["DTFINALPERIODO"]
        ));

        foreach ($data['data'] as &$cda) {
            if (($cda['IDSERVPREST'] == 'ATIVIDADE_CLIENTE' || $cda['IDSERVPREST'] == 'RELACIONAMENTO_CLIENTE') && empty($cda['CDINTECLIE']) && !empty($cda['NMINTCLIENTE'])) {
                $cda['CDINTECLIE'] = $this->CdaFactory->getCodInterlocutorCliente(array(
                    "CDCLIENTE"    => $cda["CDCLIENTE"],
                    "NMINTECLIE"   => $cda["NMINTCLIENTE"]
                ));
            } elseif ($cda['IDSERVPREST'] == 'VISITA_PROSPECT' && empty($cda['CDINTEPROS']) && !empty($cda['NMINTCLIENTE'])) {
                $cda['CDINTEPROS'] = $this->CdaFactory->getCodInterlocutorProspect(array(
                    "CDPROSPECT"    => $cda["CDPROSPECT"],
                    "NMINTEPROS"   => $cda["NMINTCLIENTE"]
                ));
            }
        }

        return $data;
    }

    public function getFuncionarios()
    {
        return $this->CdaFactory->getFuncionarios();
    }

    public function getPermissaoOperadorCda()
    {
        return $this->CdaFactory->getPermissaoOperadorCda(array(
            "CDOPERADOR"    => $this->Environment->getCdOperador(),
            "NRORG"         => $this->Environment->getNrorg(),
        ));
    }

    public function getPeriodoApuraPonto()
    {
        /**
         * Código do tipo de admissão do funcionário relacionado ao operador
         * logado.
         */
        $infoFunc = $this->CdaFactory->getTipoAdmissao(array(
            "CDOPERADOR" => $this->Environment->getCdOperador(),
        ));

        // $dtdeslfunc = null;
        // if (!empty($infoFunc)) {
        //     $dtdeslfunc = $infoFunc['DTDESLFUNC'];
        // }

        // $stdtatual = $dtdeslfunc;
        // if (is_null($dtdeslfunc)) {
        //     $dtatual = new \DateTime;
        //     $dtatual->modify('- 1 day');
        //     $stdtatual = $dtatual->format('d/m/Y');
        // }
        
        $cdtpadmissao = 'C';
        if (!empty($infoFunc['CDTPADMISSAO'])) {
            $cdtpadmissao = $infoFunc['CDTPADMISSAO'];
        }
    
        return $this->CdaFactory->getPeriodoApuraPonto(array(
            "CDTPADMISSAO"  => $cdtpadmissao,
            "NRORG"         => $this->Environment->getNrorg(),
        ));
    }

    public function getClienteCda($parametros)
    {
        return $this->CdaFactory->getClienteCda(array(
            "DTSERVPREST" => $parametros["DTSERVPREST"],
            "CDFUNC"      => $parametros["CDFUNC"],
        ));
    }

    public function getClienteRelacionamento()
    {
        $clientes = $this->CdaFactory->getClienteRelacionamento();
        foreach ($clientes['data'] as $key => $cliente) {
            if ($cliente['IDCLIEATIV'] === 'S') {
                $cliente['IDCLIEATIV'] = 0;
            } else {
                $cliente['IDCLIEATIV'] = 1;
            }
            $clientes['data'][$key] = $cliente;
        }

        return $clientes;
    }

    public function getPropostaCda($params)
    {
        $nrorg = $this->Environment->getNrorg();
        $dataParams = array(
            "CDCLIENTE" => isset($params["CDCLIENTE"]) ? $params["CDCLIENTE"] : null,
            "CDFUNC"    => isset($params["CDFUNC"]) ? $params["CDFUNC"] : null,
            "NRORG"     => $nrorg
        );
        return $this->CdaFactory->getPropostaCda($dataParams);
    }

    public function getProjetoOperadorCliente($params)
    {
        $cdoperador = $this->Environment->getCdOperador();
        $nrorg = $this->Environment->getNrorg();

        $dataParams = array(
            "CDCLIENTE"     => isset($params["CDCLIENTE"]) ? $params["CDCLIENTE"] : null,
            "CDOPERADOR"    => $cdoperador,
            "NRORG"         => $nrorg,
        );
        return $this->CdaFactory->getProjetoOperadorCliente($dataParams);
    }

    public function getTipoAtividadeCda($params)
    {
        $dataParams = array(
            "CDCLIENTE"     => isset($params["CDCLIENTE"]) ? $params["CDCLIENTE"] : null,
            "NRORG"         => isset($params["NRORG"]) ? $params["NRORG"] : null,
            "CDFUNC"        => isset($params["CDFUNC"]) ? $params["CDFUNC"] : null,
            "NRVERPROPFAS"  => isset($params["NRVERPROPFAS"]) ? $params["NRVERPROPFAS"] : null,
            "NRPROPOSFAS"   => isset($params["NRPROPOSFAS"]) ? $params["NRPROPOSFAS"] : null,
            "DTSERVPREST"   => isset($params["DTSERVPREST"]) ? $params["DTSERVPREST"] : null
        );

        return $this->CdaFactory->getTipoAtividadeCda($dataParams);
    }

    public function getProspectCda()
    {
        return $this->CdaFactory->getProspectCda();
    }

    public function getInterlocutorProspectCda($params)
    {
        $dataParams = array(
            "CDPROSPECT" => isset($params["CDPROSPECT"]) ? $params["CDPROSPECT"] : null
        );
        return $this->CdaFactory->getInterlocutorProspectCda($dataParams);
    }

    public function getInterlocutorClienteCda($params)
    {
        $dataParams = array(
            "CDCLIENTE" => isset($params["CDCLIENTE"]) ? $params["CDCLIENTE"] : null,
        );
        return $this->CdaFactory->getInterlocutorClienteCda($dataParams);
    }

    public function getLicencaUsoCda($params)
    {
        $dataParams = array(
            "CDCLIENTE" => isset($params["CDCLIENTE"]) ? $params["CDCLIENTE"] : null,
        );
        return $this->CdaFactory->getLicencaUsoCda($dataParams);
    }

    public function getMotivoEvoContCda()
    {
        return $this->CdaFactory->getMotivoEvoContCda();
    }

    public function getContatosParceiroCda($params)
    {
        return $this->CdaFactory->getContatosParceiroCda(array(
            "NRPARCNEGOCIO" => isset($params["NRPARCNEGOCIO"]) ? $params["NRPARCNEGOCIO"] : null,
        ));
    }

    public function getMotivoContatoCda()
    {
        return $this->CdaFactory->getMotivoContatoCda();
    }

    public function getSistemaCda()
    {
        return $this->CdaFactory->getSistemaCda();
    }

    public function gravaContatoCDA($parametrosContato, $parametrosCda)
    {
        $nrorg = $this->Environment->getNrorg();
        $cdoperador = $this->Environment->getCdOperador();

        try {
            $contato = '';
            $novoCodigoNrseqevolcont = '';

            if (is_null($parametrosCda["NRCONTATO"]) || $parametrosCda["NRCONTATO"] == '') {
                // Gravação na tabela CONTATO
                $novoCodigo = $this->newCode->getCodeByTableName("CONTATO", $nrorg, 1, 10, true);

                $this->CdaFactory->inserirContatoCda(array(
                    "NRCONTATO"         => $novoCodigo,
                    "CDMOTICONT"        => $parametrosContato["CDMOTICONT"],
                    "CDCLIENTE"         => $parametrosCda["CDCLIENTE"],
                    "CDPROSPECT"        => $parametrosCda["CDPROSPECT"],
                    "DTHRCONTATO"       => $parametrosCda["DTSERVPREST"],
                    "DSREFECONT"        => ($parametrosCda["IDSERVPREST"] == 'RELACIONAMENTO_CLIENTE') ? "Relacionamento com o cliente" : "Visita a Prospect",
                    "IDSTATUSCONT"      => 'A',
                    "IDTPCONTATO"       => ($parametrosCda["IDSERVPREST"] == 'RELACIONAMENTO_CLIENTE') ? 'C' : 'P',
                    "CDSISTEMA"         => $parametrosContato["CDSISTEMA"],
                    "CDOPERCONT"        => $cdoperador,
                    "NRORG"             => $nrorg,
                    "NRORGINCLUSAO"     => $nrorg,
                    "NRORGULTATU"       => $nrorg,
                    "CDOPERINCLUSAO"    => $cdoperador,
                    "NRPARCNEGOCIO"     => $parametrosContato["NRPARCNEGOCIO"],
                    "NRSEQCONTATO"      => $novoCodigo,
                ));
                $contato = $novoCodigo;
            } else {
                $this->CdaFactory->atualizarContatoCda(array(
                    "NRCONTATO"         => $parametrosCda["NRCONTATO"],
                    "CDMOTICONT"        => $parametrosContato["CDMOTICONT"],
                    "CDCLIENTE"         => $parametrosCda["CDCLIENTE"],
                    "CDPROSPECT"        => $parametrosCda["CDPROSPECT"],
                    "DSREFECONT"        => ($parametrosCda["IDSERVPREST"] == 'RELACIONAMENTO_CLIENTE') ? "Relacionamento com o cliente" : "Visita a Prospect",
                    "IDTPCONTATO"       => ($parametrosCda["IDSERVPREST"] == 'RELACIONAMENTO_CLIENTE') ? 'C' : 'P',
                    "CDSISTEMA"         => $parametrosContato["CDSISTEMA"],
                    "CDOPERCONT"        => $cdoperador,
                    "CDOPERULTATU"      => $cdoperador,
                    "NRORG"             => $nrorg,
                    "NRORGULTATU"       => $nrorg,
                    "NRPARCNEGOCIO"     => $parametrosContato["NRPARCNEGOCIO"],
                    "NRSEQCONTATO"      => $parametrosCda["NRCONTATO"],
                ));
            }

            if ($parametrosCda["IDSERVPREST"] == 'RELACIONAMENTO_CLIENTE') {
                $nrrelacionaparc = $this->CdaFactory->getNrrelacionaparcCliente(array(
                    "CDCLIENTE"     => $parametrosCda["CDCLIENTE"],
                    "CDINTECLIE"    => $parametrosCda["CDINTECLIE"],
                    "NRORG"         => $nrorg
                ));
            } else {
                $nrrelacionaparc = $this->CdaFactory->getNrrelacionaparcProspect(array(
                    "CDPROSPECT"    => $parametrosCda["CDPROSPECT"],
                    "CDINTEPROS"    => $parametrosCda["CDINTEPROS"],
                    "NRORG"         => $nrorg
                ));
            }

            //Gravação na tabela EVOCONT
            if (is_null($parametrosCda["NRSEQEVOLCONT"]) || $parametrosCda["NRSEQEVOLCONT"] == '') {
                $evolcontprox = $this->CdaFactory->consultaProxSequencialEvolCont(array(
                    'NRORG'        => $nrorg,
                    'NRCONTATO'    => ($parametrosCda['NRCONTATO'] === null) ? $contato : $parametrosCda['NRCONTATO'],
                ));

                $novoCodigoNrseqevolcont = $this->newCode->getCodeByTableName("EVOLCONT", $nrorg, 1, 10, true);

                $this->CdaFactory->inserirEvoContCda(array(
                    "NRCONTATO"         => ($parametrosCda['NRCONTATO'] === null) ? $contato : $parametrosCda['NRCONTATO'],
                    "CDTIPOCONT"        => ($parametrosCda["DTHREVOLCONT"] === null) ? '60' : null, // '60' - Visita
                    "CDPROSPECT"        => $parametrosCda["CDPROSPECT"],
                    "CDINTEPROS"        => $parametrosCda["CDINTEPROS"],
                    "CDCLIENTE"         => $parametrosCda["CDCLIENTE"],
                    "CDINTECLIE"        => $parametrosCda["CDINTECLIE"],
                    "IDSENEVOCONT"      => ($parametrosCda["DTHREVOLCONT"] === null) ? 'S' : null, // 'S' - Saída
                    "IDTPEVOLCONT"      => ($parametrosCda["IDSERVPREST"] == 'RELACIONAMENTO_CLIENTE') ? 'C' : 'P',
                    "DSEVOLCONT"        => $parametrosContato["DSEVOLCONT"],
                    "IDDESCPROT"        => 'N',
                    "CDFASECONT"        => ($parametrosCda["DTHREVOLCONT"] === null) ? '036' : null, // '036' - Visita comercial
                    "IDAGEEVOCONT"      => 'N',
                    "CDOPEREVOL"        => $cdoperador,
                    "NRSEQEVOLCON"      => $evolcontprox,
                    "CDMOTIEVOCON"      => $parametrosContato["CDMOTIEVOCON"],
                    "NRORG"             => $nrorg,
                    "NRORGINCLUSAO"     => $nrorg,
                    "CDOPERINCLUSAO"    => $cdoperador,
                    "NRORGULTATU"       => $nrorg,
                    "CDOPERULTATU"      => $cdoperador,
                    "NRRELACIONAPARC"   => $nrrelacionaparc['NRRELACIONAPARC'],
                    "IDANEXOEVOLCONT"   => 'N',
                    "TXEVOLCONT"        => $parametrosContato["DSEVOLCONT"],
                    "IDATIVO"           => 'S',
                    "NRSEQEVOLCONT"     => $novoCodigoNrseqevolcont,
                    "NRSEQCONTATO"      => ($parametrosCda['NRCONTATO'] === null) ? $contato : $parametrosCda['NRCONTATO'],
                    "IDGERAFATURA"      => 'NAO',
                    "IDEVOLPUBLICO"     => 'S',
                    "IDRESPNOTA"        => 'R',
                    "IDEVOLINTERNA"     => 'S',
                    "DTHRINCLEVOCONT"   => $parametrosCda["DTSERVPREST"],
                ));
            } else {
                $this->CdaFactory->atualizarEvoLContCda(array(
                    "NRCONTATO"         => ($parametrosCda['NRCONTATO'] === null) ? $contato : $parametrosCda['NRCONTATO'],
                    "CDPROSPECT"        => $parametrosCda["CDPROSPECT"],
                    "CDINTEPROS"        => $parametrosCda["CDINTEPROS"],
                    "CDCLIENTE"         => $parametrosCda["CDCLIENTE"],
                    "CDINTECLIE"        => $parametrosCda["CDINTECLIE"],
                    "IDTPEVOLCONT"      => ($parametrosCda["IDSERVPREST"] == 'RELACIONAMENTO_CLIENTE') ? 'C' : 'P',
                    "DSEVOLCONT"        => $parametrosContato["DSEVOLCONT"],
                    "CDOPEREVOL"        => $cdoperador,
                    "CDMOTIEVOCON"      => $parametrosContato["CDMOTIEVOCON"],
                    "NRORG"             => $nrorg,
                    "NRORGULTATU"       => $nrorg,
                    "CDOPERULTATU"      => $cdoperador,
                    "NRRELACIONAPARC"   => $nrrelacionaparc['NRRELACIONAPARC'],
                    "TXEVOLCONT"        => $parametrosContato["DSEVOLCONT"],
                    "NRSEQEVOLCONT"     => $parametrosCda["NRSEQEVOLCONT"],
                    "NRSEQCONTATO"      => ($parametrosCda['NRCONTATO'] === null) ? $contato : $parametrosCda['NRCONTATO'],
                    "DTHRINCLEVOCONT"   => $parametrosCda["DTSERVPREST"],
                ));
            }

            $parametroContatoEvolcont = array(
                "NRSEQCONTATO"  => ($parametrosCda['NRCONTATO'] === null) ? $contato : $parametrosCda['NRCONTATO'],
                "NRSEQEVOLCONT" => ($novoCodigoNrseqevolcont === '') ? $parametrosCda["NRSEQEVOLCONT"] : $novoCodigoNrseqevolcont,
            );

            return $this->CdaFactory->getEvoltcontCda($parametroContatoEvolcont);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function getDefaultFieldValues($idservprest)
    {
        $defaultvalues = array(
            "CDTIPOSERV"    => null,
            "CDCLIENTE"     => null,
        );

        if ($idservprest === "ATIVIDADE_INTERNA" || $idservprest === "FALTA_JUSTIFICADA" ||
            $idservprest === "FALTA_NAO_JUSTIFICADA" || $idservprest === "AFASTAMENTO_INSS" ||
            $idservprest === "FERIAS" || $idservprest === "VISITA_PROSPECT" ||  $idservprest === "RELACIONAMENTO_CLIENTE"
        ) {
            $defaultvalues["CDTIPOSERV"] = "030";
            if ($idservprest != "RELACIONAMENTO_CLIENTE") {
                $defaultvalues["CDCLIENTE"]  = "J26269316000177";
            }
        }
        return $defaultvalues;
    }

    public function formatarHorario($horario)
    {
        $horarioSemCaracteres = FindService::onlyNumber($horario);

        if (strlen($horarioSemCaracteres) == 4) {
            $hora = substr($horarioSemCaracteres, 0, 2);
            $minuto = substr($horarioSemCaracteres, 2, 2);
            return $hora . ':' . $minuto;
        }
        return null;
    }

    public function inserirAtividadeCda($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->Environment->getNrorg();
            $cdoperador = $this->Environment->getCdOperador();
            $valoresPadrao = $this->getDefaultFieldValues($parametros["IDSERVPREST"]);
            $validaCDAatual = $this->existenciaCdaAtual($parametros, $valoresPadrao);

            if ($validaCDAatual > 0) {
                $novaCda = false;
            } else {
                $novaCda = true;
            }

            // Criação do array da tabela SERVPREST que será usado em comum para a inserção e atualização

            $parametrosCda = array(
                "CDEMPRESA"           => $parametros["CDEMPRESA"],
                "CDFUNC"              => $parametros["CDFUNC"],
                "CDCLIENTE"           => isset($parametros["CDCLIENTE"]) ? $parametros["CDCLIENTE"] : $valoresPadrao['CDCLIENTE'],
                "DTSERVPREST"         => $parametros["DTSERVPREST"],
                "NRSERVPREST"         => $parametros["NRSERVPREST"],
                "CDTIPOSERV"          => isset($parametros["CDTIPOSERV"]) ? $parametros["CDTIPOSERV"] : $valoresPadrao['CDTIPOSERV'],
                "DSREFERENCIA"        => isset($parametros["DSREFERENCIA"]) ? $parametros["DSREFERENCIA"] : null,
                "IDSERVPREST"         => isset($parametros["IDSERVPREST"]) ? $parametros["IDSERVPREST"] : null,
                "IDSITSERVPREST"      => isset($parametros["IDSITSERVPREST"]) ? $parametros["IDSITSERVPREST"] : null,
                "CDINTECLIE"          => isset($parametros["CDINTECLIE"]) ? $parametros["CDINTECLIE"] : null,
                "DTHRENTR1TUR"        => $parametros["DTSERVPREST"],
                "DTHRSAID1TUR"        => $parametros["DTSERVPREST"],
                "DTHRENTR2TUR"        => $parametros["DTSERVPREST"],
                "DTHRSAID2TUR"        => $parametros["DTSERVPREST"],
                "NMINTCLIENTE"        => isset($parametros["NMINTCLIENTE"]) ? $parametros["NMINTCLIENTE"] : null,
                "NRPROPOSFAS"         => isset($parametros["NRPROPOSFAS"]) ? $parametros["NRPROPOSFAS"] : null,
                "NRVERPROPFAS"        => isset($parametros["NRVERPROPFAS"]) ? $parametros["NRVERPROPFAS"] : null,
                "NRSEQAGESERV"        => isset($parametros["NRSEQAGESERV"]) ? $parametros["NRSEQAGESERV"] : null,
                "NRSERVCONT"          => isset($parametros["NRSERVCONT"]) ? $parametros["NRSERVCONT"] : null,
                "CDFILIAL"            => null,
                "CDPROJETO"           => isset($parametros["CDPROJETO"]) ? ($parametros["CDPROJETO"]) : null,
                "NRORG"               => $nrorg,
                "NRORGINCLUSAO"       => $nrorg,
                "CDOPERINCLUSAO"      => $cdoperador,
                "NRORGULTATU"         => $nrorg,
                "CDOPERULTATU"        => $cdoperador,
                "CDPROSPECT"          => isset($parametros["CDPROSPECT"]) ? $parametros["CDPROSPECT"] : null,
                "NRMOTIVOEXCEFREQ"    => isset($parametros["NRMOTIVOEXCEFREQ"]) ? $parametros["NRMOTIVOEXCEFREQ"] : null,
                "QTHRTOTAL"           => isset($parametros["QTHRTOTAL"]) ? FindService::onlyNumber($parametros["QTHRTOTAL"]) : null,
                "QTHRATIVIDADEFAT"    => isset($parametros["QTHRATIVIDADEFAT"]) ? FindService::onlyNumber($parametros["QTHRATIVIDADEFAT"]) : null,
                "QTHRDESLOCAMENTOFAT" => isset($parametros["QTHRDESLOCAMENTOFAT"]) ? FindService::onlyNumber($parametros["QTHRDESLOCAMENTOFAT"]) : null,
                "DTENTRADA"           => isset($parametros['DTENTRADA']) ? $parametros["DTSERVPREST"] . ' ' . $this->formatarHorario($parametros['DTENTRADA']) : null,
                "HRAUSENCIAALMOCO"    => isset($parametros["HRAUSENCIAALMOCO"]) ? FindService::onlyNumber($parametros["HRAUSENCIAALMOCO"]) : null,
                "DTSAIDA"             => isset($parametros['DTSAIDA']) ? $parametros["DTSERVPREST"] . ' ' . $this->formatarHorario($parametros['DTSAIDA']) : null,
                "NRCONTATO"           => isset($parametros["NRSEQCONTATO"]) ? $parametros["NRSEQCONTATO"] : null,
                "DTHREVOLCONT"        => null,
                "CDINTEPROS"          => isset($parametros["CDINTEPROS"]) ? $parametros["CDINTEPROS"] : null,
                "DTHRENTR1FAT"        => $parametros["DTSERVPREST"],
                "DTHRSAID1FAT"        => $parametros["DTSERVPREST"],
                "DTHRENTR2FAT"        => $parametros["DTSERVPREST"],
                "DTHRSAID2FAT"        => $parametros["DTSERVPREST"],
                "NRLICENCAUSO"        => isset($parametros["NRLICENCAUSO"]) ? $parametros["NRLICENCAUSO"] : null,
                "NRSEQEVOLCONT"       => isset($parametros["NRSEQEVOLCONT"]) ? $parametros["NRSEQEVOLCONT"] : null,
                "DSJUSTHRNAOFATURADA" => isset($parametros["DSJUSTHRNAOFATURADA"]) ? $parametros["DSJUSTHRNAOFATURADA"] : null,
            );
    
            if (is_null($parametros["NRSERVPREST"]) || $parametros["NRSERVPREST"] == '' || $parametros["NRSERVPREST"] == '00000') {
                $nrservprest = $this->CdaFactory->consultaProxNrServPrest(array(
                    "NRORG"         => $nrorg,
                    "CDEMPRESA"     => $parametros["CDEMPRESA"],
                    "CDFUNC"        => $parametros["CDFUNC"],
                    "CDCLIENTE"     => isset($parametros["CDCLIENTE"]) ? $parametros["CDCLIENTE"] : $valoresPadrao['CDCLIENTE'],
                    "DTSERVPREST"   => $parametros["DTSERVPREST"]
                ));

                // Atualizando o NRSERVPREST tanto no $parametros - que são os dados puros vindo dos front, quanto no $parametrosCda que é a construção para o insert.
                $parametros['NRSERVPREST'] = $nrservprest;
                $parametrosCda['NRSERVPREST'] = $nrservprest;
            }

            if ($parametros["IDSERVPREST"] == 'ATIVIDADE_CLIENTE') {
                if ($novaCda) {
                    unset($parametrosCda['CLIENTEORIG']);
                    $this->CdaFactory->inserirAtividadeCda($parametrosCda);
                } else {
                    unset($parametrosCda['NRORGINCLUSAO'], $parametrosCda['CDOPERINCLUSAO']);

                    $parametrosCda['NRSERVPRESTNOVO'] = $parametros['NRSERVPREST'];
                    $parametrosCda['CLIENTEORIG'] = $parametros['CLIENTEORIG'];

                    if ($parametros['CLIENTEORIG'] !== $parametros['CDCLIENTE']) {
                        $nrservprest = $this->CdaFactory->consultaProxNrServPrest(array(
                            "NRORG"         => $nrorg,
                            "CDEMPRESA"     => $parametros["CDEMPRESA"],
                            "CDFUNC"        => $parametros["CDFUNC"],
                            "CDCLIENTE"     => isset($parametros["CDCLIENTE"]) ? $parametros["CDCLIENTE"] : $valoresPadrao['CDCLIENTE'],
                            "DTSERVPREST"   => $parametros["DTSERVPREST"]
                        ));
                        $parametrosCda['NRSERVPRESTNOVO'] = $nrservprest;
                    }
                    $this->CdaFactory->atualizarAtividadeCda($parametrosCda);
                }

                if (!is_null($parametros['PARTICIPANTES'])) {
                    $this->inserirOuAlterarParticipantes($parametros);
                }

                $this->inserirOuAlterarHorasNaoFaturadas($parametros);

                $this->inserirOuAlterarDescricao($parametros, $novaCda);

                // Busca o modelo de documento ativo
                $nrModeloDocCda = $this->CdaFactory->getModeloDocumentoCda();

                if (!is_null($nrModeloDocCda)) {
                    $this->inserirOuAlterarServPrestDoc($parametros, $nrModeloDocCda);
                }

            } elseif ($parametros["IDSERVPREST"] == 'VISITA_PROSPECT' || $parametros["IDSERVPREST"] == 'RELACIONAMENTO_CLIENTE') {
                $tiposervdefault = $this->getDefaultFieldValues($parametros["IDSERVPREST"]);

                $parametrosContato = array(
                    "CDMOTICONT"    => $parametros["CDMOTICONT"],
                    "CDMOTIEVOCON"  => $parametros["CDMOTIEVOCON"],
                    "CDSISTEMA"     => $parametros["CDSISTEMA"],
                    "CDTIPOSERV"    => $tiposervdefault["CDTIPOSERV"],
                    "NRPARCNEGOCIO" => $parametros["NRPARCNEGOCIO"],
                    "DSEVOLCONT"    => $parametros["DSEVOLCONT"]
                );

                $dadosContato = $this->gravaContatoCDA($parametrosContato, $parametrosCda);

                $parametrosCda['DTHREVOLCONT'] = $dadosContato["DTHREVOLCONT"];
                $parametrosCda['NRCONTATO'] = ($parametrosCda['NRCONTATO'] === null) ? $dadosContato["NRSEQCONTATO"] : $parametrosCda['NRCONTATO'];
                $parametrosCda['NRSEQEVOLCONT'] = $dadosContato["NRSEQEVOLCONT"];

                
                // verificar se já existe para  os casos de edição em uma CDA gerada no momento...
                if ($novaCda) {
                    $this->CdaFactory->inserirAtividadeCda($parametrosCda);
                } else {
                    unset($parametrosCda['NRORGINCLUSAO'], $parametrosCda['CDOPERINCLUSAO']);

                    $parametrosCda['NRSERVPRESTNOVO'] = $parametros['NRSERVPREST'];
                    $parametrosCda['CLIENTEORIG'] = $parametros['CLIENTEORIG'];

                    if ($parametros["IDSERVPREST"] == 'RELACIONAMENTO_CLIENTE' && $parametros['CLIENTEORIG'] !== $parametros['CDCLIENTE']) {
                        $nrservprest = $this->CdaFactory->consultaProxNrServPrest(array(
                            "NRORG"         => $nrorg,
                            "CDEMPRESA"     => $parametros["CDEMPRESA"],
                            "CDFUNC"        => $parametros["CDFUNC"],
                            "CDCLIENTE"     => isset($parametros["CDCLIENTE"]) ? $parametros["CDCLIENTE"] : $valoresPadrao['CDCLIENTE'],
                            "DTSERVPREST"   => $parametros["DTSERVPREST"]
                        ));
                        $parametrosCda['NRSERVPRESTNOVO'] = $nrservprest;
                    }

                    $this->CdaFactory->atualizarAtividadeCda($parametrosCda);
                }

                if ($parametros["IDSERVPREST"] == 'VISITA_PROSPECT') {
                    $this->CdaFactory->atualizarInfoInterlocutorProspect(array(
                        "NRTEL1INPROS"    => $parametros["NRTEL1INPROS"],
                        "DSEMAILIPROS"    => $parametros["DSEMAILIPROS"],
                        "CDPROSPECT"      => $parametros["CDPROSPECT"],
                    ));
                } elseif ($parametros["IDSERVPREST"] == 'RELACIONAMENTO_CLIENTE') {
                    $this->CdaFactory->atualizarInfoInterlocutorCliente(array(
                        "NRTEL1INCLIE"    => $parametros["NRTEL1INCLIE"],
                        "DSEMAILICLIE"    => $parametros["DSEMAILICLIE"],
                        "CDINTECLIE"      => $parametros["CDINTECLIE"],
                    ));
                }
            } elseif ($parametros["IDSERVPREST"] == 'ATIVIDADE_INTERNA' || $parametros["IDSERVPREST"] == 'FERIAS' || $parametros["IDSERVPREST"] == 'FALTA_JUSTIFICADA' || $parametros["IDSERVPREST"] == 'AFASTAMENTO_INSS' || $parametros["IDSERVPREST"] == 'FALTA_NAO_JUSTIFICADA') {
                // verificar se já existe para  os casos de edição em uma CDA gerada no momento...
                if ($novaCda) {
                    $this->CdaFactory->inserirAtividadeCda($parametrosCda);
                } else {
                    unset($parametrosCda['NRORGINCLUSAO'], $parametrosCda['CDOPERINCLUSAO']);
                    $this->CdaFactory->atualizarAtividadeCda($parametrosCda);
                }
            } elseif ($parametros['IDSERVPREST'] === 'FALTA_NAO_JUSTIFICADA') {
                if ($novaCda) {
                    $this->ProcedureService->executarFpaCdaTotaliza($nrorg, $parametrosCda["CDEMPRESA"], $parametrosCda["CDFUNC"], $parametrosCda["DTSERVPREST"]);
                }
            }

            $retorno = (array(
                "NRSERVPREST"   => $parametros["NRSERVPREST"],
                "DTSERVPREST"   => $parametros["DTSERVPREST"],
                "CDCLIENTE"     => isset($parametros["CDCLIENTE"]) ? $parametros["CDCLIENTE"] : $valoresPadrao['CDCLIENTE'],
                "is_new"        => $novaCda,
            ));

            $this->connection->commit();
            
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
        return $retorno;
    }

    public function inserirOuAlterarServPrestDoc($parametros, $nrModeloDocCda)
    {
        try {
            $nrorg = $this->Environment->getNrorg();
            $cdoperador = $this->Environment->getCdOperador();
            $valoresPadrao = $this->getDefaultFieldValues($parametros["IDSERVPREST"]);

            $valida = $this->CdaFactory->verificaModeloDocJaInserido(array(
                "CDEMPRESA"           => $parametros["CDEMPRESA"],
                "CDFUNC"              => $parametros["CDFUNC"],
                "CDCLIENTE"           => isset($parametros["CDCLIENTE"]) ?  $parametros["CDCLIENTE"] : $valoresPadrao['CDCLIENTE'],
                "DTSERVPREST"         => $parametros["DTSERVPREST"],
                "NRSERVPREST"         => $parametros["NRSERVPREST"],
            ));

            if (count($valida)> 0){
               $nrServPrestDoc =  $valida[0]['NRSERVPRESTDOC'];
               $inserindo = false;
            }
            else {
               $nrServPrestDoc =  $this->newCode->getCodeByTableName("SERVPRESTDOC", $nrorg, 1, 10, true);
               $inserindo = true;
            }

            $dadosDoc = array(
                "NRSERVPRESTDOC"      => $nrServPrestDoc,
                "CDEMPRESA"           => $parametros["CDEMPRESA"],
                "CDFUNC"              => $parametros["CDFUNC"],
                "CDCLIENTE"           => isset($parametros["CDCLIENTE"]) ?  $parametros["CDCLIENTE"] : $valoresPadrao['CDCLIENTE'],
                "DTSERVPREST"         => $parametros["DTSERVPREST"],
                "NRSERVPREST"         => $parametros["NRSERVPREST"],
                "NRMODELODOCCDA"      => $nrModeloDocCda,
                "NRORG"               => $nrorg,
                "NRORGINCLUSAO"       => $nrorg,
                "CDOPERINCLUSAO"      => $cdoperador,
                "NRORGULTATU"         => $nrorg,
                "CDOPERULTATU"        => $cdoperador,
            );

            //O valor vindo da query é um float, para isso, transformamos ele em inteiro
            if ($inserindo === true) {
                unset($dadosDoc['NRORGULTATU'], $dadosDoc['CDOPERULTATU']);
                $this->CdaFactory->inserirServPrestDoc($dadosDoc);
            } else {
                unset($dadosDoc['NRORGINCLUSAO'], $dadosDoc['CDOPERINCLUSAO']);
                $this->CdaFactory->alterarServPrestDoc($dadosDoc);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function inserirOuAlterarParticipantes($parametros)
    {
        try {
            $nrorg = $this->Environment->getNrorg();
            $cdoperador = $this->Environment->getCdOperador();
            $valoresPadrao = $this->getDefaultFieldValues($parametros["IDSERVPREST"]);

            foreach ($parametros['PARTICIPANTES'] as $participante) {

                // Verifica se o participante jÃ¡ foi inserido
                $valida = $this->CdaFactory->verificarParticipanteJaInserido(array(
                    "CDEMPRESA" => $parametros["CDEMPRESA"],
                    "CDFUNC" => $parametros["CDFUNC"],
                    "CDCLIENTE" => $parametros["CDCLIENTE"],
                    "DTSERVPREST" => $parametros["DTSERVPREST"],
                    "NRSERVPREST" => $parametros["NRSERVPREST"],
                    "PARTICIPANTE" => $participante["DSNOMEPARTICIP"]
                ));

                if (count($valida) > 0) {
                    $nrServPrestParticip =  $participante["NRSERVPRESTPARTICIP"];
                    $inserindo = false;
                }
                else {
                    $nrServPrestParticip =  $this->newCode->getCodeByTableName("SERVPRESTPARTICIP", $nrorg, 1, 10, true);
                    $inserindo = true;
                }

                $dadosParticipante = array(
                    "NRSERVPRESTPARTICIP" => $nrServPrestParticip,
                    "DSNOMEPARTICIP"      => $participante["DSNOMEPARTICIP"],
                    "DSSETORPARTICIP"     => isset($participante["DSSETORPARTICIP"]) ? $participante["DSSETORPARTICIP"] : null,
                    "CDEMPRESA"           => $parametros["CDEMPRESA"],
                    "CDFUNC"              => $parametros["CDFUNC"],
                    "CDCLIENTE"           => isset($parametros["CDCLIENTE"]) ?  $parametros["CDCLIENTE"] : $valoresPadrao['CDCLIENTE'],
                    "DTSERVPREST"         => $parametros["DTSERVPREST"],
                    "NRSERVPREST"         => $parametros["NRSERVPREST"],
                    "NRORG"               => $nrorg,
                    "NRORGINCLUSAO"       => $nrorg,
                    "CDOPERINCLUSAO"      => $cdoperador,
                    "NRORGULTATU"         => $nrorg,
                    "CDOPERULTATU"        => $cdoperador,
                );


                if ($inserindo === true) {
                    unset($dadosParticipante['NRORGULTATU'], $dadosParticipante['CDOPERULTATU']);
                    $this->CdaFactory->inserirParticipantesCda($dadosParticipante);
                } else {
                    unset($dadosParticipante['NRORGINCLUSAO'], $dadosParticipante['CDOPERINCLUSAO']);
                    $this->CdaFactory->atualizarParticipantes($dadosParticipante);
                }
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function inserirOuAlterarDescricao($parametros, $novaCda)
    {
        try {
            $nrorg = $this->Environment->getNrorg();
            $cdoperador = $this->Environment->getCdOperador();

            $paramsDesc = array(
                "DSSERVPREST"         => $parametros["DESCSERVPRESS"],
                "CDEMPRESA"           => $parametros["CDEMPRESA"],
                "CDFUNC"              => $parametros["CDFUNC"],
                "CDCLIENTE"           => $parametros["CDCLIENTE"],
                "DTSERVPREST"         => $parametros["DTSERVPREST"],
                "NRSERVPREST"         => $parametros["NRSERVPREST"],
                "NRORG"               => $nrorg,
                "NRORGINCLUSAO"       => $nrorg,
                "CDOPERINCLUSAO"      => $cdoperador,
                "NRORGULTATU"         => $nrorg,
                "CDOPERULTATU"        => $cdoperador,
            );

        
            if ($novaCda) {
                unset($paramsDesc['NRORGULTATU'], $paramsDesc['CDOPERULTATU']);
                $this->CdaFactory->inserirDescricaoCDA($paramsDesc);
            } else {
                $haDesc = $this->getDescricaoCda(array(
                    "CDEMPRESA"           => $parametros["CDEMPRESA"],
                    "CDFUNC"              => $parametros["CDFUNC"],
                    "CDCLIENTE"           => $parametros["CDCLIENTE"],
                    "DTSERVPREST"         => $parametros["DTSERVPREST"],
                    "NRSERVPREST"         => $parametros["NRSERVPREST"],
                    "NRORG"               => $nrorg,
                ));
        
                if (count($haDesc) > 0) {
                    unset($paramsDesc['CDOPERINCLUSAO'], $paramsDesc['CDOPERINCLUSAO']);
                    $this->CdaFactory->atualizarDescricaoCDA($paramsDesc);
                } else {
                    $this->CdaFactory->inserirDescricaoCDA($paramsDesc);
                }
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getDescricaoCda($parametros)
    {
        try {
            $nrorg = $this->Environment->getNrorg();

            $data = $this->CdaFactory->getDescricaoCda(array(
                "CDEMPRESA"           => $parametros["CDEMPRESA"],
                "CDFUNC"              => $parametros["CDFUNC"],
                "CDCLIENTE"           => $parametros["CDCLIENTE"],
                "DTSERVPREST"         => $parametros["DTSERVPREST"],
                "NRSERVPREST"         => $parametros["NRSERVPREST"],
                "NRORG"               => $nrorg,
            ));

            foreach ($data as &$row) {
                $row['DSSERVPREST'] = mb_check_encoding($row['DSSERVPREST'], 'UTF-8') ? $row['DSSERVPREST'] :  utf8_encode($row['DSSERVPREST']);
                $row['DSSERVPREST'] = str_replace("", "-", $row['DSSERVPREST']);
                $row['DSSERVPREST'] = str_replace("", "Є", $row['DSSERVPREST']);
            }
            return $data;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function saveDescricaoAtividadeCda($parametros)
    {
        $paramsDesc = array(
            "DESCSERVPRESS"       => $parametros["DSSERVPREST"],
            "CDEMPRESA"           => $parametros["CDEMPRESA"],
            "CDFUNC"              => $parametros["CDFUNC"],
            "CDCLIENTE"           => $parametros["CDCLIENTE"],
            "DTSERVPREST"         => $parametros["DTSERVPREST"],
            "NRSERVPREST"         => $parametros["NRSERVPREST"],
        );

        $this->inserirOuAlterarDescricao($paramsDesc, false);
    }

    public function getParticipantesCda($parametros)
    {
        return $this->CdaFactory->getParticipantesCda(array(
            "CDEMPRESA"           => $parametros["CDEMPRESA"],
            "CDFUNC"              => $parametros["CDFUNC"],
            "CDCLIENTE"           => $parametros["CDCLIENTE"],
            "DTSERVPREST"         => $parametros["DTSERVPREST"],
            "NRSERVPREST"         => $parametros["NRSERVPREST"],
        ));
    }

    public function deleteParticipantes($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $this->CdaFactory->deleteParticipantes(FindService::formatarArray2($parametros['NRSERVPRESTPARTICIP']));
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function inserirOuAlterarHorasNaoFaturadas($parametros)
    {
        try {
            $nrorg = $this->Environment->getNrorg();
            $cdoperador = $this->Environment->getCdOperador();
            $valoresPadrao = $this->getDefaultFieldValues($parametros["IDSERVPREST"]);

            foreach ($parametros['HORASNFATURADAS'] as $horaNFatu) {

                 // Verifica se o participante já foi inserido
                 $valida = $this->CdaFactory->verificarHorasNFatuInserido(array(
                    "CDEMPRESA" => $parametros["CDEMPRESA"],
                    "CDFUNC" => $parametros["CDFUNC"],
                    "CDCLIENTE" => $parametros["CDCLIENTE"],
                    "DTSERVPREST" => $parametros["DTSERVPREST"],
                    "NRSERVPREST" => $parametros["NRSERVPREST"],
                    "NRJUSTHORASNAOFATUCDA" => $horaNFatu["NRJUSTHORASNAOFATUCDA"]
                ));

                if (count($valida) > 0) {
                    $nrServPrestJustHrNFat =  $valida[0]["NRSERVPRESTJUSTHRNFAT"];
                    $inserindo = false;
                }
                else {
                    $nrServPrestJustHrNFat =  $this->newCode->getCodeByTableName("SERVPRESTJUSTHRNFAT", $nrorg, 1, 10, true);
                    $inserindo = true;
                }

                $hora_sem_dois_pontos = str_replace(":", "", $horaNFatu["QTHORASNAOFATU"]);
                $hora_formatada = str_pad($hora_sem_dois_pontos, 4, '0', STR_PAD_LEFT);

                $dados = array(
                    "NRSERVPRESTJUSTHRNFAT"     => $nrServPrestJustHrNFat,
                    "CDEMPRESA"                 => $parametros["CDEMPRESA"],
                    "CDFUNC"                    => $parametros["CDFUNC"],
                    "CDCLIENTE"                 => isset($parametros["CDCLIENTE"]) ?  $parametros["CDCLIENTE"] : $valoresPadrao['CDCLIENTE'],
                    "DTSERVPREST"               => $parametros["DTSERVPREST"],
                    "NRSERVPREST"               => $parametros["NRSERVPREST"],
                    "QTHORASNAOFATU"            => $hora_formatada,
                    "NRJUSTHORASNAOFATUCDA"     => $horaNFatu["NRJUSTHORASNAOFATUCDA"],
                    "NRORG"                     => $nrorg,
                    "CDOPERINCLUSAO"            => $cdoperador,
                    "NRORGINCLUSAO"             => $nrorg,
                    "NRORGULTATU"               => $nrorg,
                    "CDOPERULTATU"              => $cdoperador,
                );

                if ($inserindo === true) {
                    unset($dados['NRORGULTATU'], $dados['CDOPERULTATU']);
                    $this->CdaFactory->inserirHorasNaoFaturadas($dados);
                } else {
                    unset($dados['NRORGINCLUSAO'], $dados['CDOPERINCLUSAO']);
                    $this->CdaFactory->atualizarHorasNaoFaturadas($dados);
                }
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function gerarDocumento($parametros)
    {
        // parametros = dados da cda e idoma do documento
        $dadosDocumento = $this->CdaFactory->buscarDocumento(array(
            "CDEMPRESA"           => $parametros["CDEMPRESA"],
            "CDFUNC"              => $parametros["CDFUNC"],
            "CDCLIENTE"           => $parametros["CDCLIENTE"],
            "DTSERVPREST"         => $parametros["DTSERVPREST"],
            "NRSERVPREST"         => $parametros["NRSERVPREST"],
            "CDIDIOM"             => $parametros["CDIDIOM"],
        ));
    }

    public function getHorasNaoFaturadas($parametros)
    {
        return $this->CdaFactory->getHorasNaoFaturadas(array(
            "CDEMPRESA"           => $parametros["CDEMPRESA"],
            "CDFUNC"              => $parametros["CDFUNC"],
            "CDCLIENTE"           => $parametros["CDCLIENTE"],
            "DTSERVPREST"         => $parametros["DTSERVPREST"],
            "NRSERVPREST"         => $parametros["NRSERVPREST"],
        ));
    }

    public function existenciaCda($parametros)
    {
        return $this->CdaFactory->existenciaCda(array(
            "DTSERVPREST"         => $parametros["DTSERVPREST"],
        ));
    }

    public function existenciaCdaAtual($parametros, $valoresPadrao)
    {
        $dados = $this->CdaFactory->existenciaCdaAtual(array(
            "CDEMPRESA"           => $parametros["CDEMPRESA"],
            "CDFUNC"              => $parametros["CDFUNC"],
            "CDCLIENTE"           => isset($parametros["CDCLIENTE"]) ? $parametros["CDCLIENTE"] : $valoresPadrao['CDCLIENTE'],
            "DTSERVPREST"         => $parametros["DTSERVPREST"],
            "NRSERVPREST"         => $parametros["NRSERVPREST"],
        ));
        $resultado = $dados[0]["EXISTE"];

        return $resultado; 
    }

    public function confirmaOuCorrigeCda($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $nrorg = $this->Environment->getNrorg();
            $cdoperador = $this->Environment->getCdOperador();
            
            if (($parametros['ACAO'] == 'corrigir') && ($parametros['IDDOCUMCERTSIGN'] != null)){

                /* Exclui o documento no portal de Assinaturas */
                $this->assinaturaCDAService->excluirdocumento($parametros['IDDOCUMCERTSIGN']);

                /* Exclui os signatários da CDA */
                $this->CdaFactory->limpaSignatariosAssinaturaCDA(array(
                    "NRSERVPRESTDOC" => $parametros['NRSERVPRESTDOC'],
                    "NRORG"          => $nrorg
                ));

                /* Limpa os campos às assinaturas */
                $this->CdaFactory->limpaCamposAssinaturaCDA(array(
                    "NRSERVPRESTDOC" => $parametros['NRSERVPRESTDOC'],
                    "NRORG"          => $nrorg
                ));
            }


            $this->CdaFactory->confirmaOuCorrigeCda(array(
                "CDEMPRESA"                 => $parametros["CDEMPRESA"],
                "CDFUNC"                    => $parametros["CDFUNC"],
                "CDCLIENTE"                 => $parametros["CDCLIENTE"],
                "DTSERVPREST"               => $parametros["DTSERVPREST"],
                "NRSERVPREST"               => $parametros["NRSERVPREST"],
                "IDSITSERVPREST"            => ($parametros['ACAO'] === 'confirmar') ?  'C' : 'P',
                "NRORG"                     => $nrorg,
                "NRORGULTATU"               => $nrorg,
                "CDOPERULTATU"              => $cdoperador,
            ));

            $this->connection->commit();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function deletarCda($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $nrorg = $this->Environment->getNrorg();
            foreach ($parametros as $value) {
                $dados = array(
                    "CDEMPRESA"                 => $value["CDEMPRESA"],
                    "CDFUNC"                    => $value["CDFUNC"],
                    "CDCLIENTE"                 => $value["CDCLIENTE"],
                    "DTSERVPREST"               => $value["DTSERVPREST"],
                    "NRSERVPREST"               => $value["NRSERVPREST"],
                );

                $this->CdaFactory->deleteBatidaPonto($dados);
                $this->CdaFactory->deleteDescServPrest($dados);
                $this->CdaFactory->deleteServPrestDoc($dados);
                
                if ($value['IDSERVPREST'] === 'ATIVIDADE_CLIENTE') {
                    $this->CdaFactory->deleteParticipantesCda($dados);
                    $this->CdaFactory->deleteHorasNaoFaturadas($dados);
                } elseif ($value['IDSERVPREST'] === 'VISITA_PROSPECT' || $value['IDSERVPREST'] === 'RELACIONAMENTO_CLIENTE') {
                    $this->CdaFactory->deleteEvolCont(array(
                        "NRSEQEVOLCONT" => $value['NRSEQEVOLCONT'],
                        "NRCONTATO"     => $value['NRCONTATO'],
                        "DTHREVOLCONT"  => $value['DTHREVOLCONT'],
                    ));
                } elseif ($value['IDSERVPREST'] === 'FALTA_NAO_JUSTIFICADA') {
                    $this->ProcedureService->executarFpaCdaTotaliza($nrorg, $value["CDEMPRESA"], $value["CDFUNC"], $value["DTSERVPREST"]);
                }

                $this->CdaFactory->deleteServPrest($dados);
            }
            
            $this->connection->commit();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function inserirRepresentante($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $nrorg = $this->Environment->getNrorg();
            $this->CdaFactory->inserirRepresentante(array(
                "CDINTECLIE"    => $this->newCode->getCodeByTableName("INTECLIE".$parametros["CDCLIENTE"], $nrorg, 1, 10, true),
                "CDCLIENTE"     => $parametros['CDCLIENTE'],
                "NMINTECLIE"    => $parametros['NMINTECLIE'],
                "NRCPFICLIE"    => $parametros['NRCPFICLIE'],
                "DSEMAILICLIE"  => $parametros['DSEMAILICLIE'],
                "NRORG"         => $nrorg,
                "NRORGINCLUSAO" => $nrorg
            ));
        
            $this->connection->commit();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function assinarDocumentoManualmente($parametros)
    {

        /* 1 - Alterar o registro dos signatários marcados para assinado 
        
           2 - Verificar se todos os signatários do documento estão assinados. Se sim, alterar a CDA para assinado.
           
           OBS: Deve ser verificado se o campo "IDDOCUMCERTSIGN" esta preenchido. Se estiver, deve ser feito este processo para todas as cdas vinculadas com este mesmo "IDDOCUMCERTSIGN" */
        $this->connection->beginTransaction();
        try {
            $nrorg = $this->Environment->getNrorg();
            $operador = $this->Environment->getCdOperador();

            foreach ($parametros['CDAS'] as $cda) {
               
                if ($cda['IDDOCUMCERTSIGN'] != null){

                    $where = 'AND NRSERVPRESTDOC IN (SELECT NRSERVPRESTDOC FROM SERVPRESTDOC WHERE IDDOCUMCERTSIGN = '.$cda['IDDOCUMCERTSIGN'].')';

                }
                else {
                    $where = 'AND NRSERVPRESTDOC = '.$cda['NRSERVPRESTDOC'];
                }

                foreach($parametros['REPRESENTANTE'] as $representante) {

                    if ($cda['IDSITASSINATURA'] == 'NAO_ENVIADO'){

                        $novoCodigo = $this->newCode->getCodeByTableName("SERVPRESTDOCASSINA", $nrorg, 1, 10, true);

                        $params = array (
                            "NRSERVPRESTDOCASSINA" => $novoCodigo,
                            "NRSERVPRESTDOC" => $cda['NRSERVPRESTDOC'],
                            "NRCPFSIGNATARIO" => $representante["NRCPFICLIE"],
                            "NMSIGNATARIO" => $representante["NMINTECLIE"],
                            "DSEMAILSIGNATARIO" => $representante["DSEMAILICLIE"],
                            "IDTIPOASSINATURA" => 'E',
                            "NRORG" => $nrorg,
                            "OPERADOR" => $operador
                        );

                        $this->CdaFactory->criaSiginatarioManual($params);
                    } 

                    $params = array (
                        "DSJUSTIFMANUALASSINAT" => $parametros['DSJUSTIFMANUALASSINAT'],
                        "OPERADOR"  => $operador,
                        "NRORG"   => $nrorg,
                        "NRCPFSIGNATARIO" => $representante['NRCPFICLIE']
                    );

                    $this->CdaFactory->assinarManualmenteSignatario($params, $where);
                }

                $valida = $this->CdaFactory->validaAssinaturasManuais($where);
                
                $valida = $valida["data"];

                if ((($valida[0]['QTMANUAL'] != 0) && ($valida[0]['QTREGISTROS'] != 0)) && ($valida[0]['QTMANUAL'] == $valida[0]['QTREGISTROS'])){

                    $params = array (
                        "IDSITASSINATURA" => 'MANUAL'
                    );

                    $this->CdaFactory->assinarManualmenteCDA($params, $where);
                }
                else {

                    $params = array (
                        "IDSITASSINATURA" => 'PARCIAL'
                    );

                    $this->CdaFactory->assinarManualmenteCDA($params, $where);
                }
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            throw $e;
        }

    } 

    public function recebeEmailOperador($operCDA, $operador){

        $params = array (
            "CDOPERADOR" => $operCDA
        );

        $operAux = $this->CdaFactory->recebeEmailOperador($params);

        if (count($operAux) > 0){
            $emailOper = $operAux[0]['DSEMAILOPER'];
        }
        else {
            $params = array (
                "CDOPERADOR" => $operador
            );
    
            $operAux = $this->CdaFactory->recebeEmailOperador($params);

            if (count($operAux)> 0){
                $emailOper = $operAux[0]['DSEMAILOPER'];
            }
            else {
                $emailOper = null; 
            }
        }

        return $emailOper;

    }

    public function enviarDocumentoParaAssinatura($parametros)
    {   
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->Environment->getNrorg();
            $operador = $this->Environment->getCdOperador();
           
            /* Gera documento para envio */
            $paramsDoc = array( 
                "CDA"       => $parametros['CDAS'],
                "IDIOMA"    => $parametros['IDIOMA']
            );
            
            $nmDocumento = 'x';
            
            foreach ($parametros['CDAS'] as $cda){
                if ($nmDocumento == 'x'){
                    $nmDocumento = str_replace('-', '_', $cda['CODIGO']);
                    $nmDocumento = str_replace('/', '_', $nmDocumento); 
                } else {
                    $nmDocumento = $nmDocumento.'-'.str_replace('/', '_', $cda['DATA']);
                }
            }

            $nmDocumento = $nmDocumento.'.pdf';

            $doc = $this->DocumentoCdaService->mergeDocCda($paramsDoc);

            $documento = $this->tempDirFS . DIRECTORY_SEPARATOR . $operador . '.pdf';

            /* Envia documento  para o Certisign */
            $upload = $this->assinaturaCDAService->uploadtoCertsing($nmDocumento, $documento);

            //debugApi([$upload]);
            $uploadId = $upload["response"]["uploadId"];
           
            /*  Cria o documento no Certsign */
            
            /* -- Recebe dados do Sender -- */
            $params= array(
                "NRORG" => $nrorg
            );        
            
            $responsavel = $this->CdaFactory->getDadosSender($params);
            
            $sender = array();
             
            foreach ($responsavel as $resp) {
                $sender['NOME']  = $resp['NMSENDERCERTSIGN'];
                $sender['EMAIL'] = $resp['DSEMAILSENDERCERTSIGN'];
                $sender['CPF']   = $resp['NRCPFSENDERCERTSIGN'];
            }

            /* Recede parametrizações  do documento */
            $tipoDoc = (int)$responsavel[0]['NRTIPODOCCERTSIGN'];
            $padraoAssinatura = (int)$responsavel[0]['IDPADRAOASSINACERTSIGN'];


            $emailOperResp = $this->recebeEmailOperador($parametros['CDAS'][0]['CDOPERADOR'], $operador);
            //$emailOperResp ='arthur.tondato@teknisa.com'; 

            $signers = array();
            /* -- Recebe os Signatários Digitais --   

            Por hora, só estão sendo utilizados os Signatários Eletrônicos. 
              
            $signers = array();

            foreach ($signatarios as $ind => $sign) {
                
                $signers[$ind] = array(
                    'POSICAO' => $sign['NRORDENACAO'],
                    'TIPO'    => 'Signer',
                    'NOME'    => $sign['NOME'],
                    'EMAIL'   => $sign['EMAIL'],
                    'CPF'     => $sign['NRCPFSIGNATARIO']
                );
            }
            */
            /*  -- Recebe os Signatários Eletrônicos -- */
            
            $eletrSigners = array();
        
            foreach ($parametros['REPRESENTANTES'] as $ind => $sign) {
            
                $eletrSigners[$ind] = array(
                    'POSICAO' => 1,
                    'TIPO'    => 'Signer',
                    'NOME'    => $sign['NMINTECLIE'],
                    'EMAIL'   => $sign['DSEMAILICLIE'],
                    'CPF'     => $sign['NRCPFICLIE']
                );
            }
            
            /* -- Chama a API para criar o Documento no Certsign -- */
            $docCertsign = $this->assinaturaCDAService->createSignatureCertsing($uploadId, $nmDocumento, $sender, $signers, $eletrSigners, $padraoAssinatura, $tipoDoc, $emailOperResp);
           
            //debugApi([$docCertsign]);
            //inserir no log do create
            // $this->createCertisignNoLog($docCertsign, $nrseqorcamento, $nrtipoorcamento, $nrorg);
            
            //if ($docCertsign["response"]["message"]){

            /* -- Atualiza o registro dos documentos da proposta para incluir o ID e a Chave do Certsing -- */
            
            $id    = $docCertsign['response']['id'];
            $chave = $docCertsign['response']['chave'];
            $attendees = $docCertsign['response']['attendees'];

            foreach ($parametros['CDAS'] as $cda) {

                $params = array(
                    "IDDOCUMCERTSIGN"    => $id,
                    "CHAVEDOCUMCERTSIGN" => $chave,
                    "OPERADOR"           => $operador,   
                    "NRORG"              => $nrorg,
                    "NRSERVPRESTDOC"     => $cda["NRSERVPRESTDOC"]
                );        
                
                $this->CdaFactory->updateDocumentoIdCertsign($params);
                
                /* -- Atualiza o registro dos signatários da proposta para incluir o ID do Certsing -- */

                foreach ($attendees as $signatario){
                    
                        $nrDocAssinat = $this->newCode->getCodeByTableName("SERVPRESTDOCASSINA", $nrorg, 1, 10, true);
                    
                        $params = array(
                            "NRSERVPRESTDOCASSINA" => $nrDocAssinat,
                            "NRSERVPRESTDOC"       => $cda["NRSERVPRESTDOC"],
                            "NRCPFSIGNATARIO"      => $signatario["individualIdentificationCode"],
                            "NMSIGNATARIO"         => $signatario["name"],
                            "DSEMAILSIGNATARIO"    => $signatario["email"],
                            "IDSIGNATCERTSIGN"     => $signatario['signerId'],
                            "DSURLASSINATURA"      => $signatario['signUrl'],
                            "NRORG"                => $nrorg,
                            "OPERADOR"             => $operador
                        );
                        
                        $this->CdaFactory->insereDocAssinatura($params);
                } 
            }
            $this->connection->commit();
            return "OK";
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getDadosSignatarioCDA($parametros)
    {
        $dataParams = array(
            "CDCLIENTE" => isset($parametros["CDCLIENTE"]) ? $parametros["CDCLIENTE"] : null,
        );
        return $this->CdaFactory->getDadosSignatarioCDA($dataParams);
    }

    public function salvarDadosSignatarioCDA($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->Environment->getNrorg();
            $operador = $this->Environment->getCdOperador();

            foreach ($parametros["DRG"] as $row) {
                if ($row["CDINTECLIE"] != 0) {
                    $parametrosRepresentanteEdit = array (
                        "CDINTECLIE"    => $row['CDINTECLIE'],
                        "CDCLIENTE"     => $row['CDCLIENTE'],
                        "NRCPFICLIE"    => $row['NRCPFICLIE'],
                        "DSEMAILICLIE"  => $row['DSEMAILICLIE'],
                        "NRORGULTATU"   => $nrorg,
                        "CDOPERULTATU"  => $operador
                    );

                    $this->CdaFactory->editarRepresentante($parametrosRepresentanteEdit);
                } else {
                    $cdintclie = $this->newCode->getCodeByTableName("INTECLIE".$row["CDCLIENTE"], $nrorg, 1, 10, true);

                    $parametrosRepresentanteNew = array(
                        "CDINTECLIE"    => $cdintclie,
                        "CDCLIENTE"     => $row['CDCLIENTE'],
                        "NMINTECLIE"    => $row['NMINTECLIE'],
                        "NRCPFICLIE"    => $row['NRCPFICLIE'],
                        "DSEMAILICLIE"  => $row['DSEMAILICLIE'],
                        "NRORG"         => $nrorg,
                        "NRORGINCLUSAO" => $nrorg
                    );
                    
                    $this->CdaFactory->inserirRepresentante($parametrosRepresentanteNew);
                }
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getDadosAssinaturaCDA($parametros)
    {
        $nrorg = $this->Environment->getNrorg();

        return $this->CdaFactory->getDadosAssinaturaCDA(array(
            "NRSERVPRESTDOC"  => isset($parametros["NRSERVPRESTDOC"])? $parametros["NRSERVPRESTDOC"]: null,
            "NRORG"           => $nrorg,
        ));
    } 

    public function getDadosAssinaturaCDAAssManual($parametros)
    {
        $nrorg = $this->Environment->getNrorg();

        return $this->CdaFactory->getDadosAssinaturaCDAAssManual(array(
            "NRSERVPRESTDOC"  => isset($parametros["NRSERVPRESTDOC"])? $parametros["NRSERVPRESTDOC"]: null,
            "NRORG"           => $nrorg,
        ));
    } 

    public function getDocsRelacionadosCDA($parametros)
    {
        $nrorg = $this->Environment->getNrorg();

        return $this->CdaFactory->getDocsRelacionadosCDA(array(
            "IDDOCUMCERTSIGN"   => isset($parametros["IDDOCUMCERTSIGN"])? $parametros["IDDOCUMCERTSIGN"]: null,
            "NRORG"             => $nrorg,
        ));
    }
    
    public function criarLancamentosRecorrentes($parametros)
    {
        try {
            foreach ($parametros["ROWS"] as &$row) {
                $row["NRMOTIVOEXCEFREQ"] = $parametros["NRMOTIVOEXCEFREQ"];
                $row["IDSERVPREST"] = $parametros["IDSERVPREST"];
                $row["NRSERVPREST"] = "00000";
                $array = [$row];
                $this->inserirAtividadeCda($array[0]);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }
}
