<?php

namespace SM\Util;

use Teknisa\Libs\Service\CRUD as CRUDService;
use Teknisa\Libs\Util\Environment;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\Libs\Util\DateUtil;

class Utilities
{

    protected $crudService;
    protected $environment;
    protected $address;
    protected $confirmaEnderecoService;
    const SERVICES_ENTITY_DIR = '\Teknisa\Model\Services\Entities\\';
    public function __construct(CRUDService $crudService, Environment $environment, $address, $confirmaEnderecoService)
    {
        $this->crudService = $crudService;
        $this->environment = $environment;
        $this->address = $address;
        $this->confirmaEnderecoService = $confirmaEnderecoService;
    }

    /**
     * Valida no banco se o dados que estao sendo inseridos ja não existem nos campos dos quais estão sendo gravados
     * Retona true se nao existir ou false se já existir
     * @param string  $string
     * @param string  $string
     * @param Array  $string
     */
    public function validateDuplicatedRecords($row, $dataSourceName, $pkColumns, $validationColumns)
    {
        $conditions = array();

        foreach ($validationColumns as $column) {
            $value = $row[$column];
            array_push($conditions, array(
                "columnName" => $column,
                "operator"   => is_array($value) ? "IN" : ((!empty($value)) ? '=' : "IS_NULL"),
                "value"      => $value
            ));
        }

        if (!in_array("NRORG", $validationColumns)) {
            array_push($conditions, array(
                "columnName" => "NRORG",
                "operator"   => "=",
                "value"      => $this->environment->getNrOrg()
            ));
        }

        if ($pkColumns) {
            foreach ($pkColumns as $column) {
                $value = $row[$column];
                array_push($conditions, array(
                    "columnName" => $column,
                    "operator"   => "<>",
                    "value"      => $value ? $value : -1
                ));
            }
        }

        $dataSet = $this->crudService->getFilteredDataSet($dataSourceName, $conditions);
        if (!empty($dataSet->getRows())) {
            return false;
        } else {
            return true;
        }
    }

    /**
     * Recebe uma string com valores separados por um delimitador, e retorna um array
     * a partir dessa string.
     *
     * @param string  $valor
     * @param boolean $int
     * @param string  $delimitador
     */
    public static function getArrayDeString($valor, $int = false, $delimitador = ",")
    {
        if (empty($valor)) {
            $value = array();
        } else {
            if ($int) {
                $value = array_map("intval", explode($delimitador, $valor));
            } else {
                $value = explode($delimitador, $valor);
            }
        }

        return $value;
    }

    /**
     * Recebe um objeto DateTime e retorna o nome do mês em português.
     */
    public static function getNomeMesPortugues($data)
    {
        $mes = $data->format("m");
        $nome = null;

        switch ($mes) {
            case "01":
                $nome = "Janeiro";
                break;

            case "02":
                $nome = "Fevereiro";
                break;

            case "03":
                $nome = "Março";
                break;

            case "04":
                $nome = "Abril";
                break;

            case "05":
                $nome = "Maio";
                break;

            case "06":
                $nome = "Junho";
                break;

            case "07":
                $nome = "Julho";
                break;

            case "08":
                $nome = "Agosto";
                break;

            case "09":
                $nome = "Setembro";
                break;

            case "10":
                $nome = "Outubro";
                break;

            case "11":
                $nome = "Novembro";
                break;

            case "12":
                $nome = "Dezembro";
                break;

            default:
                $nome = null;
        };

        return $nome;
    }

    /**
     * Formata um número para a exibição.
     *
     * @param integer $numero
     * @param integer $casasDecimais
     *
     * @return string
     */
    public static function formatarNumero($numero, $casasDecimais = 2, $separadorDecimal = ",", $separadorMilhar = ".")
    {
        if (!is_numeric($numero)) {
            $numero = 0.0;
        }
        return number_format($numero, $casasDecimais, $separadorDecimal, $separadorMilhar);
    }

    /**
     * Aplica a máscara de CPF ou CNPJ em uma string.
     *
     * @param string $str
     */
    public static function formatarCpfCnpj($str)
    {
        if (strlen($str) == 14) {
            return self::mascararString("##.###.###/####-##", $str);
        } elseif (strlen($str) == 11) {
            return self::mascararString("###.###.###-##", $str);
        } else {
            return $str;
        }
    }

    /**
     * Aplica a máscara de CEP em uma string.
     *
     * @param string $str
     */
    public static function formatarCep($str)
    {
        if (strlen($str) == 8) {
            return self::mascararString("##.###-###", $str);
        } else {
            return $str;
        }
    }

    /**
     * Aplica uma máscara a uma string.
     *
     * @param string $mask Máscara em "#" (ex: CPF: ###.###.###-##)
     * @param string $str String a ser formatada
     */
    public static function mascararString($mask, $str)
    {
        $str = str_replace(" ", "", $str);

        for ($i = 0; $i < strlen($str); $i++) {
            $mask[strpos($mask, "#")] = $str[$i];
        }

        return $mask;
    }

    public static function uploadArquivoFileServer($zhFileUploader, $customFolder, $filePath)
    {
        if (is_file($filePath)) {
            $size = filesize($filePath);
            $fileName = basename($filePath);
            $fileContent = file_get_contents($filePath);
            $type = null;

            switch (strtolower(substr(strrchr($fileName, "."), 1))) {
                case "pdf": {
                        $type = "application/pdf";
                        break;
                }
                case "docx": {
                        $type = "application/vnd.openxmlformats-officedocument.wordprocessingml.document";
                        break;
                }
                case "gif": {
                        $type = "image/gif";
                        break;
                }
                case "png": {
                        $type = "image/png";
                        break;
                }
                case "jpg": {
                        $type = "image/jpg";
                        break;
                }
                case "txt": {
                        $type = "text/plain";
                        break;
                }
            }

            return self::uploadFileServer($zhFileUploader, $customFolder, $fileName, $fileContent, $size, $type);
        }
        return null;
    }

    public static function uploadFileServer($zhFileUploader, $customFolder, $fileName, $fileContent, $size, $type)
    {
        $base64Content = base64_encode($fileContent);

        $fileData = array(
            'b64File' => 'data:' . $type . ';base64,' . $base64Content,
            'name' => $fileName,
            'size' => $size,
            'type' => $type,
            'webkitRelativePath' => ''
        );

        $row = new \Zeedhi\Framework\DTO\Row(array('FILE' => array($fileData)));
        $zhFileUploader->uploadFilesInRow($row, 'FILE', $customFolder);

        return $row["FILE"][0];
    }

    public function findAddress(Row $row, Response $response)
    {
        $this->address->findAddress($row, $response);
        if (!empty($response->getDataSets())) {
            $foundAddress = $response->getDataSets()[0]->getRows();
            $foundAddress = $this->confirmaEnderecoService->confirmAddress($foundAddress);
            if (!empty($foundAddress['MESSAGES'])) {
                $response->addMessage(new Message($foundAddress['MESSAGES']));
            }

            $response->addDataSet(new DataSet('ENDERECO', $foundAddress['ADDRESS']));
        }
    }

    public static function setDataSetDefault(Request $request, Response $response, array $dataSet)
    {
        $response->addDataset(new DataSet(explode('/', $request->getRoutePath())[1], $dataSet));
    }

    /**
     * Converta parte hora em decimal
     * @static
     *
     * @param $hora
     *
     * @return float
     */
    public static function converteHoraDecimal($hora)
    {
        $aux = floor($hora);

        if ($aux === $hora) {
            return $hora;
        } else {
            $minuto = $hora - $aux;
            $decimal = $minuto * 100.0 / 60.0;
            $aux += $decimal;

            return $aux;
        }
    }

    public static function adicionarMes($data, $qtdMeses)
    {
        $novaData = DateUtil::adicionaIntervalo($data, $qtdMeses, DateUtil::MES);
        if ($data->format("m/Y") == $novaData->format("m/Y")) {
            $novaData = DateUtil::adicionaIntervalo($novaData, 3);
            $novaData = DateUtil::getPrimeiroDiaDoMes($novaData);
        }

        return $novaData;
    }

    public static function getCondicaoIn999(array $itens, $nomeColuna, $isString = false)
    {
        if (count($itens) > 999) {
            $novoArray = array_chunk($itens, 999);
        } else {
            $novoArray = array($itens);
        }

        $itensString = array();
        foreach ($novoArray as $value) {
            if ($isString) {
                $itensString[] = "'" . implode("','", $value) . "'";
            } else {
                $itensString[] = implode(",", $value);
            }
        }

        $primeiroInElementos = $itensString[0];
        unset($itensString[0]);

        $condicao = " ($nomeColuna IN ($primeiroInElementos) ";
        foreach ($itensString as $item) {
            $condicao .= " 
            OR $nomeColuna IN ($item) ";
        }

        return $condicao . ") ";
    }

    /**
     * Converte um número do formato string para float.
     * Pode estar tanto no formado americano quanto brasileiro.
     *
     * @param $numero
     * @return float
     */
    public static function stringToFloat($numero)
    {
        if (is_string($numero) && strlen($numero) > 0) {
            $numero = str_replace(",", ".", $numero);
            $pos = strrpos($numero, ".");
            if ($pos !== false) {
                $numero = str_replace(".", "", substr($numero, 0, $pos)) . substr($numero, $pos);
                $numero = floatval($numero);
            }
        }

        return $numero;
    }

    /**
     * Gera o template teknisa com a mensagem do email
     **/
    public static function getTemplateEmailTeknisa(array $dadosSubstituiTemplate)
    {
        $template = new \Teknisa\SM\Util\Mail\Templates\Teknisa\TemplateTeknisa();
        return $template->setValoresEmail($dadosSubstituiTemplate);
    }

    public static function downloadArquivoFileServer($fileUrl, $fileName, $fileType, $fileSize)
    {
        header('Accept-Ranges: none');
        header('Content-Type: ' . $fileType);
        //header('Content-Length: ' . $fileSize);
        header('Content-Disposition: inline; filename="' . $fileName . '"');

        ob_clean();
        flush();
        readfile($fileUrl);

        exit;
    }

    public static function decodificarDynamicFilter($dynamicFilter)
    {
        return empty($dynamicFilter) ? null : json_decode(utf8_encode(base64_decode($dynamicFilter)), true);
    }

    public static function rtf2textPlain($text, $ignoreException = true, $useUtf8Encode = false)
    {
        if (!strlen($text) || substr($text, 0, 5) != '{\rtf') {
            if ($ignoreException) {
                $ansa = mb_check_encoding($text, 'UTF-8') ? $text :  utf8_encode($text);
            } else {
                throw new \Exception('Invalid RTF!');
            }
        } else {
            // we'll try to fix up the parts of the rtf as best we can
            // clean up the file a little to simplify parsing
            $text = str_replace("\r", ' ', $text); // returns
            $text = str_replace("\n", ' ', $text); // new lines
            $text = str_replace('  ', ' ', $text); // double spaces
            $text = str_replace('} {', '}{', $text); // embedded spaces
            // skip over the heading stuff
            $j = strpos($text, '{', 1); // skip ahead to the first part of the header

            $loc = 1;
            $t = "";

            $ansa = "";
            $len = strlen($text);
            self::getpgraph($text, $j, $len); // skip by the first paragrap

            while ($j < $len) {
                $c = substr($text, $j, 1);
                if ($c == "\\") {
                    // have a tag
                    $tag = self::gettag($text, $j, $len);
                    if (strlen($tag) > 0) {
                        // process known tags
                        switch ($tag) {
                            case 'par':
                                $ansa .= "\r\n";
                                break;
                                // ad a list of common tags
                                // parameter tags
                            case 'spriority1':
                            case 'fprq2':
                            case 'author':
                            case 'operator':
                            case 'sqformat':
                            case 'company':
                            case 'xmlns1':
                            case 'wgrffmtfilter':
                            case 'pnhang':
                            case 'themedata':
                            case 'colorschememapping':
                                $tt = self::gettag($text, $j, $len);
                                break;
                            case '*':
                            case 'info':
                            case 'stylesheet':
                                // gets to end of paragraph
                                $j--;
                                self::getpgraph($text, $j, $len);
                            default:
                                self::convertTagToSpecialCharacter($tag, $ansa);
                        }
                    }
                } else {
                    $ansa .= $c;
                }
                $j++;
            }
            $ansa = str_replace('{', '', $ansa);
            $ansa = str_replace('}', '', $ansa);
        }
        return $ansa;
    }

    private static function convertTagToSpecialCharacter($tag, &$ansa)
    {

        $arrayNumber = ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9'];
        $lastOccurrence = 0;

        if (substr($tag, 0, 1) == '\'') {
            $tag = str_replace('\\', '', $tag);
            $tag = str_replace('\'', '', $tag);

            //get last occurrence of the number
            foreach ($arrayNumber as $number) {
                $tmpLastPos = strrpos($tag, $number);
                if ($lastOccurrence < $tmpLastPos) {
                    $lastOccurrence = $tmpLastPos;
                }
            }

            $tagLength = strlen($tag);

            $c = substr($tag, 0, $lastOccurrence == 0 ? 2 : $lastOccurrence + 1);
            $unicodeChar = '\u';
            //create unicode char with tag code RTF
            if (strlen($c) == 1) {
                $unicodeChar = '\u000' . $c;
            } elseif (strlen($c) == 2) {
                $unicodeChar = '\u00' . $c;
            } elseif (strlen($c) == 3) {
                $unicodeChar = '\u0' . $c;
            } elseif (strlen($c) == 4) {
                $unicodeChar = '\u' . $c;
            }

            //decode unicode to ascii
            $ansa .= json_decode('"' . $unicodeChar . '"');


            //get characters after tag code rtf
            if ($tagLength > $lastOccurrence + 1) {
                $lenghtToGet = $tagLength - ($lastOccurrence + 1);
                $ansa .= substr($tag, $lastOccurrence + 1, $lenghtToGet) . " ";
            }
        }
    }


    private static function gettag(&$text, &$j, &$len)
    {
        // gets the text following the / character or gets the param if it there

        $tag = '';
        while (true) {
            $j++;
            if ($j >= $len) {
                break;
            }
            $c = substr($text, $j, 1);
            if ($c == ' ') {
                break;
            }
            if ($c == ';') {
                break;
            }
            if ($c == '}') {
                break;
            }
            if ($c == "\\") {
                $j--;
                break;
            }
            if ($c == "{") {
                break;
            }
            if ((($c >= '0') && ($c <= '9')) || (($c >= 'a') && ($c <= 'z')) || (($c >= 'A') && ($c <= 'Z')) || $c == "'" || $c == "-" || $c == "*") {
                $tag = $tag . $c;
            } else {
                $j--;
                break;
            }
        }
        return $tag;
    }

    private static function getpgraph(&$text, &$j, &$len)
    {
        // if the first char after a tag is { then throw out the entire paragraph
        // this has to be nested
        $nest = 0;
        while (true) {
            $j++;
            if ($j >= $len) {
                break;
            }
            if (substr($text, $j, 1) == '}') {
                if ($nest == 0) {
                    return;
                }
                $nest--;
            }
            if (substr($text, $j, 1) == '{') {
                $nest++;
            }
        }
    }
}
